<?php
/**
 * Sympa based group provider
 *
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author     Etienne MELEARD <etienne.meleard AT renater.fr>
 */

if(!defined('DOKU_INC')) define('DOKU_INC', realpath(dirname(__FILE__).'/../../../').'/');
if(!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN', DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'action.php');

class action_plugin_sympagroups extends DokuWiki_Action_Plugin {
    public function register(&$controller) {
        $controller->register_hook('DOKUWIKI_STARTED', 'BEFORE', $this, 'handle');
    }
    
    public function handle(&$event, $args) {
        foreach(array('enabled', 'mailSource', 'soapService', 'applicationId', 'applicationPwd') as $k) if(!$this->getConf($k)) return;
        
        if(!$_SERVER['REMOTE_USER']) {
            unset($_SESSION[DOKU_COOKIE]['sympagroups']);
            return;
        }
        if(array_key_exists('sympagroups', $_SESSION[DOKU_COOKIE])) return;
        
        global $USERINFO;
        switch($this->getConf('mailSource')) {
            case 'user' : $mail = $_SERVER['REMOTE_USER']; break;
            case 'mail' :
            default : $mail = $USERINFO['mail'];
        }
        if(!mail_isvalid($mail)) return;
        
        try {
            $soapClient = new SoapClient($this->getConf('soapService'));
        } catch(SoapFault $fault) {
            error_log('sympagroups : unable to establish a SOAP connection with '.$this->getConf('soapService'));
            return;
        }
        
        try {
            $res = $soapClient->authenticateRemoteAppAndRun(
                $this->getConf('applicationId'),
                $this->getConf('applicationPwd'),
                'USER_EMAIL='.$mail,
                'complexWhich',
                array()
            );
        } catch(SoapFault $fault) {
            msg('Dokuwiki groups only', 0);
            error_log('sympagroups : unable to authenticate on '.$this->getConf('soapService').' : '.$fault->getMessage());
            return;
        }
        if(!$res || !is_array($res)) return;
        
        $grps = array();
        foreach($res as $list) {
            $grps[] = $list->listAddress;
            if($list->isOwner == 'true') $grps[] = preg_replace('/@/', '-request@', $list->listAddress);
            if($list->isEditor == 'true') $grps[] = preg_replace('/@/', '-editor@', $list->listAddress);
        }
        $USERINFO['grps'] = $this->getConf('exclusive') ? $grps : array_unique(array_merge($USERINFO['grps'], $grps));
        $_SESSION[DOKU_COOKIE]['auth']['info']['grps'] = $USERINFO['grps'];
        $_SESSION[DOKU_COOKIE]['sympagroups'] = true;
        
        // Needed for admin area access ...
        global $INFO;
        $INFO = pageinfo();
    }
}
